// src/errors.ts
class SocketError extends Error {
  name = "SocketError";
  originalError;
  constructor(message, originalError) {
    super(message);
    this.originalError = originalError;
  }
}

class UnpackError extends SocketError {
}

// src/messages.ts
var Heartbeat = Symbol("Heartbeat");
function pack(data) {
  return JSON.stringify({ _: data });
}
function unpack(data) {
  try {
    const parsedData = JSON.parse(data);
    if (typeof parsedData === "number") {
      return Heartbeat;
    }
    if (typeof parsedData === "object" && parsedData !== null && "_" in parsedData && typeof parsedData._ !== "undefined") {
      return { data: parsedData._ };
    }
    throw new UnpackError("Unpack Error: Invalid message shape");
  } catch (error) {
    if (error instanceof Error) {
      throw new UnpackError("Unpack Error: Failed to deserialize", error);
    }
    throw error;
  }
}

// src/socket.ts
var THIRTY_SECONDS = 30 * 1000;

class ManagedSocket {
  url;
  socket;
  onMessage = () => {};
  onError = () => {};
  onConnected = () => {};
  onDisconnected = () => {};
  _reconnectAttempts = 0;
  _messageQueue = [];
  _debugMode = false;
  _offline = false;
  constructor(url, handlers = {}) {
    this.url = url;
    this.onMessage = handlers.onMessage ?? this.onMessage;
    this.onError = handlers.onError ?? this.onError;
    this.onConnected = handlers.onConnected ?? this.onConnected;
    this.onDisconnected = handlers.onDisconnected ?? this.onDisconnected;
    this.socket = this.connect();
    setInterval(() => {
      this.sendHeartbeat();
    }, THIRTY_SECONDS);
    window.addEventListener("offline", () => {
      this._offline = true;
      if (this.isConnected) {
        this.debugMessage("Internet connection lost, closing socket...");
        this.close();
      }
    });
    window.addEventListener("online", () => {
      this._offline = false;
      if (!this.isConnected) {
        this.debugMessage("Internet connection restored, reconnecting socket...");
        this.attemptReconnection();
      }
    });
  }
  sendHeartbeat() {
    if (this.socket?.readyState === WebSocket.OPEN) {
      this.socket.send(JSON.stringify(Date.now()));
    }
  }
  debugMessage(...args) {
    if (this._debugMode) {
      const output = ["[Socket]", ...args];
      console.log(...output);
    }
  }
  sendQueuedMessages() {
    const messagesToSend = [...this._messageQueue];
    this._messageQueue = [];
    for (const message of messagesToSend) {
      this.send(message);
    }
  }
  attemptReconnection() {
    if (this._offline) {
      this.debugMessage("Browser offline; skipping reconnection attempt");
      return;
    }
    const delay = Math.min(2000 ** this._reconnectAttempts, THIRTY_SECONDS);
    this.debugMessage(`Attempting to reconnect in ${delay}ms`);
    setTimeout(() => {
      this._reconnectAttempts++;
      this.connect();
    }, delay);
  }
  connect() {
    const newSocket = new WebSocket(this.url);
    newSocket.addEventListener("open", () => {
      this._reconnectAttempts = 0;
      this.onConnected();
      this.sendQueuedMessages();
    });
    newSocket.addEventListener("close", ({ wasClean, code, reason }) => {
      this.debugMessage("WebSocket closed", {
        wasClean,
        code,
        reason
      });
      this.onDisconnected({
        wasClean,
        code,
        reason
      });
      if (!this._offline) {
        this.debugMessage("Attempting automatic reconnection");
        return this.attemptReconnection();
      }
    });
    newSocket.addEventListener("error", (error) => {
      this.debugMessage("WebSocket error", error);
      const socketError = new SocketError("Socket error", new Error(error.currentTarget?.toString()));
      this.onError(socketError);
    });
    newSocket.addEventListener("message", (messageEvent) => {
      const data = unpack(messageEvent.data);
      if (data === Heartbeat) {
        return;
      }
      const { data: unpacked } = data;
      this.debugMessage("Received message", unpacked);
      this.onMessage(unpacked);
    });
    this.socket = newSocket;
    return newSocket;
  }
  get isConnected() {
    return this.socket?.readyState === WebSocket.OPEN;
  }
  debug(debugModeEnabled) {
    this._debugMode = debugModeEnabled;
  }
  close() {
    if (this.socket?.readyState !== WebSocket.OPEN) {
      throw new SocketError("Cannot close a socket that is not open");
    }
    this.socket.close();
  }
  send(data) {
    if (!this.socket || this.socket.readyState !== WebSocket.OPEN) {
      this.debugMessage("WebSocket is not open. Queuing message.");
      this._messageQueue.push(data);
      return;
    }
    this.socket.send(pack(data));
  }
}
export {
  unpack,
  pack,
  UnpackError,
  SocketError,
  ManagedSocket,
  Heartbeat
};
